from django.core.management.base import BaseCommand
from telegram.ext import Updater, CommandHandler
from ...models.config import NotifyConfig
from django.contrib.auth import get_user_model


def required_argument(fn):
    def wrapper(update, context):
        config = NotifyConfig.get_solo()
        if int(len(context.args)) == 0:
            update.message.reply_text(config.telegram_bad_command_text)
            return False
        return fn(update, context)

    return wrapper


def start(update, context):
    config = NotifyConfig.get_solo()
    update.message.reply_text(config.telegram_welcome_text)
    update.message.reply_text(config.telegram_help_text)


def show_help(update, context):
    config = NotifyConfig.get_solo()
    update.message.reply_text(config.telegram_help_text)


@required_argument
def command_set_key(update, context):
    config = NotifyConfig.get_solo()
    telegram_secret = context.args[0]
    User = get_user_model()
    user = User.objects.filter(telegram_secret=telegram_secret).first()
    if user is not None:
        user.telegram_chat_id = update.message.chat_id
        user.save()
        update.message.reply_text(config.telegram_success_added_text)
    else:
        update.message.reply_text(config.telegram_failed_added_text)


class Command(BaseCommand):
    help = 'Telegram garpix_notify daemon.'

    def handle(self, *args, **options):
        notify_config = NotifyConfig.get_solo()
        updater = Updater(notify_config.telegram_api_key)
        updater.dispatcher.add_handler(CommandHandler('start', start))
        updater.dispatcher.add_handler(CommandHandler('set', command_set_key, pass_args=True))
        updater.dispatcher.add_handler(CommandHandler('help', show_help))
        updater.start_polling()
        updater.idle()



import json
import re
from datetime import datetime
from typing import Optional, List

import requests
from django.conf import settings
from django.contrib.auth import get_user_model
from django.db import models, transaction
from django.db.models import Manager
from django.utils.html import format_html
from django.utils.module_loading import import_string
from django.db.models.signals import post_save
from django.dispatch import receiver

from .log import NotifyErrorLog
from .user_list import NotifyUserList
from .category import NotifyCategory
from .choices import TYPE, STATE
from .file import NotifyFile
from .template import NotifyTemplate
from ..exceptions import IsInstanceException
from ..mixins import UserNotifyMixin
from ..utils.send_data import SendData

from ..clients import SMSClient, EmailClient, CallClient, TelegramClient, ViberClient, PushClient, WhatsAppClient

NotifyMixin = import_string(getattr(settings, 'GARPIX_NOTIFY_MIXIN', 'garpix_notify.mixins.notify_mixin.NotifyMixin'))

User = get_user_model()


class Notify(NotifyMixin, UserNotifyMixin):
    """
    Уведомление
    """
    subject = models.CharField(max_length=255, default='', blank=True, verbose_name='Тема')
    text = models.TextField(verbose_name='Текст')
    html = models.TextField(verbose_name='HTML', blank=True, default='')

    user: User = models.ForeignKey(settings.AUTH_USER_MODEL, blank=True, null=True, on_delete=models.SET_NULL,
                                   related_name='notifies', verbose_name='Пользователь (получатель)')
    email = models.EmailField(max_length=255, blank=True, null=True, verbose_name='Email Получателя',
                              help_text='Используется только в случае отсутствия указанного пользователя')
    sender_email = models.EmailField(max_length=255, blank=True, null=True, verbose_name='Email Отправителя')

    state = models.IntegerField('Состояние', choices=STATE.CHOICES, default=STATE.WAIT)
    event = models.IntegerField('Событие', choices=settings.CHOICES_NOTIFY_EVENT, blank=True, null=True)
    room_name = models.CharField('Название комнаты', max_length=255, null=True, blank=True)
    type = models.IntegerField(choices=TYPE.CHOICES, verbose_name='Тип')
    category = models.ForeignKey(NotifyCategory, on_delete=models.CASCADE, related_name='notifies',
                                 verbose_name='Категория')

    files = models.ManyToManyField(NotifyFile, verbose_name='Файлы')

    is_read = models.BooleanField(default=False, verbose_name='Прочитано')
    data_json = models.TextField(blank=True, null=True, verbose_name='Данные пуш-уведомления (JSON)')

    users_list = models.ManyToManyField(NotifyUserList, blank=True, verbose_name='Списки пользователей для рассылки')

    send_at = models.DateTimeField(blank=True, null=True, verbose_name='Время начала отправки')
    created_at = models.DateTimeField('Дата создания', auto_now_add=True)
    sent_at = models.DateTimeField('Дата отправки', blank=True, null=True)

    objects = Manager()

    def __str__(self):
        return self.subject if self.subject and self.subject != '' else f'Уведомление № {self.id}'

    def _get_sender(self):
        if self.user:
            self.email = self.user.email if self.user.email else self.email
            self.phone = str(self.user.phone) if self.user.phone else str(self.phone)
            self.telegram_chat_id = self.user.telegram_chat_id if self.user.telegram_chat_id else self.telegram_chat_id
            self.viber_chat_id = self.user.viber_chat_id if self.user.viber_chat_id else self.viber_chat_id

        if self.phone is not None:
            self.phone = re.sub(r"\D", "", self.phone)

    def start_send(self):  # noqa

        # Если передан пользователь, то перезаписываем данные (если они есть у пользователя)
        self._get_sender()

        if self.type == TYPE.EMAIL:
            EmailClient.send_email(self)
        elif self.type == TYPE.SMS:
            SMSClient.send_sms(self)
        elif self.type == TYPE.PUSH:
            PushClient.send_push(self)
        elif self.type == TYPE.TELEGRAM:
            TelegramClient.send_telegram(self)
        elif self.type == TYPE.VIBER:
            ViberClient.send_viber(self)
        elif self.type == TYPE.CALL:
            CallClient.send_call(self)
        elif self.type == TYPE.WHATSAPP:
            WhatsAppClient.send_whatsapp(self)

        self.save()

    @staticmethod
    def send(event: int, context: dict, user: User = None, email: str = None, phone: str = None,  # noqa: C901
             files: list = None, data_json: dict = None, viber_chat_id: str = None, room_name: str = None,
             notify_templates: list = None, send_at: datetime = None, send_now: bool = False,
             user_want_message_check: bool = False, **kwargs) -> List[Optional['Notify']]:

        if user and not isinstance(user, User):
            raise IsInstanceException()

        instance_list: list = []

        # Сначала забираем те данные, которые передали с методом
        notify_user = user if user else None
        notify_email = email if email else None
        notify_phone = phone if phone else None
        notify_viber_chat_id = viber_chat_id if viber_chat_id else None

        local_context = context.copy()
        data_json = json.dumps(data_json) if data_json else None

        if notify_templates:
            templates = (
                NotifyTemplate.objects
                .select_related('category', 'user')
                .prefetch_related('user_lists')
                .filter(id__in=notify_templates, event=event, is_active=True)
            )
        else:
            templates = (
                NotifyTemplate.objects
                .select_related('category', 'user')
                .prefetch_related('user_lists')
                .filter(event=event, is_active=True)
            )

        file_instances = []
        if files:
            file_instances = list(map(lambda file: NotifyFile.objects.create(file=file), files))

        for template in templates:

            notify_users_lists = template.user_lists.all()

            # Формируем список основных получателей письма
            # Из шаблона, если методом не было передано данных
            template_user = template.user if template.user else None
            template_email = template.email if template.email else None
            template_phone = template.phone if template.phone else None
            template_viber_chat_id = template.viber_chat_id if template.viber_chat_id else None

            # Данные, которые указал пользователь у себя в профиле в приоритете
            if notify_user:
                notify_email = notify_user.email if notify_user.email else notify_email
                notify_phone = notify_user.phone if notify_user.phone else notify_phone
                notify_viber_chat_id = (
                    notify_user.viber_chat_id if notify_user.viber_chat_id else notify_viber_chat_id
                )

            if notify_user is None and template_user:
                notify_user = template.user if template.user else None
                notify_email = template.user.email if template.user.email else notify_email
                notify_phone = template.user.phone if template.user.phone else notify_phone
                notify_viber_chat_id = (
                    template.user.viber_chat_id if template.user.viber_chat_id else notify_viber_chat_id
                )

            if notify_email is None:
                notify_email = template_email

            if notify_phone is None:
                notify_phone = template_phone

            if notify_viber_chat_id is None:
                notify_viber_chat_id = template_viber_chat_id

            # Проверка, хочет ли пользователь получить сообщение
            if user_want_message_check and hasattr(
                    settings, 'NOTIFY_USER_WANT_MESSAGE_CHECK') and settings.NOTIFY_USER_WANT_MESSAGE_CHECK is not None:

                user_want_message = import_string(settings.NOTIFY_USER_WANT_MESSAGE_CHECK)

                if not notify_users_lists.exists():
                    user_check = user_want_message(event, template.type, notify_user)
                    if not user_check:
                        continue
                else:
                    # Если у нас шаблон со списками, то формируем новый из переданных пользователей,
                    # Если список пустой, то отменяем отправку
                    notify_users_lists = user_want_message(event, template.type, notify_user, notify_users_lists)
                    if not notify_users_lists:
                        continue

            # Передаем пользователя в контекст
            if notify_user is not None:
                if local_context is not None:
                    local_context.update({
                        'user': notify_user,
                    })
                else:
                    local_context = {
                        'user': notify_user,
                    }
            local_context['event_id'] = event

            if send_at is not None:
                notify_send = send_at
            else:
                notify_send = template.send_at

            instance = Notify.objects.create(
                subject=template.render_subject(local_context),
                text=template.render_text(local_context),
                html=template.render_html(local_context),
                user=notify_user,
                email=notify_email,
                phone=notify_phone if notify_phone is not None else "",
                viber_chat_id=notify_viber_chat_id if notify_viber_chat_id is not None else "",
                type=template.type,
                event=template.event,
                category=template.category if template.category else None,
                data_json=data_json,
                send_at=notify_send,
                room_name=room_name,
                **kwargs
            )
            if notify_users_lists.exists():
                instance.users_list.add(*notify_users_lists)
            file_instance = instance.files
            for f in file_instances:
                file_instance.add(f)
            instance.save()

            if send_now:
                transaction.on_commit(lambda: instance.start_send())

            instance_list.append(instance)
        return instance_list

    @staticmethod
    def call(phone: str, user: User = None, url: str = None, **kwargs) -> Optional[str]:
        call_url_type: int = CallClient.get_url_type()

        if user and not isinstance(user, User):
            raise IsInstanceException()

        if user is not None:
            phone = user.phone if user.phone else phone

        if url is not None:
            url = url
        else:
            url = SendData.call_url(call_url_type)

        main_url = url.format(to=phone, **kwargs)
        response_url = requests.get(main_url)
        response_dict: dict = response_url.json()
        value, response = CallClient.get_value_checker(response_dict)

        if value == "OK":
            return '{Code}'.format(**response)
        return None

    def to_log(self, error_text: str) -> None:
        log: NotifyErrorLog = NotifyErrorLog(notify=self, error=error_text)
        log.save()

    def get_format_state(self):
        if self.state == STATE.WAIT:
            return format_html('<span style="color:orange;">В ожидании</span>')
        elif self.state == STATE.DELIVERED:
            return format_html('<span style="color:green;">Отправлено</span>')
        elif self.state == STATE.REJECTED:
            return format_html('<span style="color:red;">Отклонено</span>')
        elif self.state == STATE.DISABLED:
            return format_html('<span style="color:red;">Отправка запрещена</span>')
        return format_html('<span style="color:black;">Неизвестный статус</span>')

    get_format_state.short_description = 'Статус'

    class Meta:
        verbose_name = 'Уведомление'
        verbose_name_plural = 'Уведомления'


@receiver(post_save, sender=Notify)
def system_post_save(sender, instance, created, **kwargs):
    from garpix_notify.tasks.tasks import send_system_notifications
    if created and instance.type == TYPE.SYSTEM:
        transaction.on_commit(lambda: send_system_notifications.delay(instance.pk))



from django.contrib import admin
from garpix_notify.models.category import NotifyCategory


@admin.register(NotifyCategory)
class NotifyCategoryAdmin(admin.ModelAdmin):
    list_display = ('title', 'created_at')
    readonly_fields = ('created_at',)



from .sms_client import SMSClient
from .email_client import EmailClient
from .telegram_client import TelegramClient
from .viber_client import ViberClient
from .push_client import PushClient
from .call_client import CallClient
from .whatsapp_client import WhatsAppClient



from django.contrib import admin, messages
from ..models.template import NotifyTemplate
from ..models.notify import Notify
from django.http import HttpResponseRedirect


@admin.register(NotifyTemplate)
class NotifyTemplateAdmin(admin.ModelAdmin):
    change_form_template = "send_notify.html"
    fields = (
        'title',
        'is_active',
        'subject',
        'get_context_description',
        'text',
        'html',
        'user',
        'email',
        'phone',
        'telegram_chat_id',
        'viber_chat_id',
        'type',
        'category',
        'event',
        'user_lists',
        'send_at',
    )
    readonly_fields = (
        'get_context_description',
        'get_event_description',
    )
    list_display = ('title', 'is_active', 'type', 'category', 'event', 'user', 'email', 'phone', 'send_at')
    list_filter = ('type', 'category', 'event', 'is_active')
    actions = ['create_mailing', ]
    filter_horizontal = ('user_lists',)

    def create_mailing(self, request, queryset):
        count = Notify.send(event=None, context={}, notify_templates=queryset)
        self.message_user(request, 'Рассылка создана, кол-во сообщений: {}'.format(count))

    create_mailing.short_description = "Сделать рассылку"

    def response_change(self, request, obj):
        from ..models.notify import Notify
        if obj.user_lists and "_send_now" in request.POST:
            context = obj.get_test_data()
            template = obj
            instance = Notify.objects.create(
                subject=obj.render_subject(template.subject),
                text=obj.render_text(context),
                html=obj.render_html(context),
                user=obj.user,
                email=obj.email,
                type=obj.type,
                category=obj.category,
            )
            instance.save()
            instance.start_send()
            return HttpResponseRedirect(".")
        return super().response_change(request, obj)

    def get_changelist(self, request, **kwargs):
        events_message = NotifyTemplate.get_blank_events_message()
        if events_message:
            self.message_user(request, events_message, level=messages.WARNING)
        return super().get_changelist(request, **kwargs)



from typing import Optional
from django.conf import settings
from django.db import DatabaseError, ProgrammingError

from garpix_notify.models.choices import CALL_URL, SMS_URL
from garpix_notify.models.config import NotifyConfig


class SendData:

    def __init__(self):
        try:
            self.config = NotifyConfig.get_solo()
            self.SMS_API_ID = self.config.sms_api_id
            self.SMS_LOGIN = self.config.sms_login
            self.SMS_PASSWORD = self.config.sms_password
            self.SMS_FROM = self.config.sms_from
        except (DatabaseError, ProgrammingError):
            self.SMS_API_ID = getattr(settings, 'SMS_API_ID', 1234567890)
            self.SMS_LOGIN = getattr(settings, 'SMS_LOGIN', '')
            self.SMS_PASSWORD = getattr(settings, 'SMS_PASSWORD', '')
            self.SMS_FROM = getattr(settings, 'SMS_FROM', '')
        try:
            self.CALL_API_ID = self.config.call_api_id
            self.CALL_LOGIN = self.config.call_login
            self.CALL_PASSWORD = self.config.call_password
        except (DatabaseError, ProgrammingError):
            self.CALL_API_ID = getattr(settings, 'CALL_API_ID', 1234567890)
            self.CALL_LOGIN = getattr(settings, 'CALL_LOGIN', '')
            self.CALL_PASSWORD = getattr(settings, 'CALL_PASSWORD', '')
            self.CALL_FROM = getattr(settings, 'CALL_FROM', '')

    def __get_call_url(self, key: int) -> Optional[str]:
        url_dict_call = {
            0: (f'{CALL_URL.SMSRU_CALL_URL}?api_id={self.CALL_API_ID}' + '&phone={to}&json=1'),
            1: (f'{CALL_URL.SMSRU_CALL_URL}?login={self.CALL_LOGIN}&password={self.CALL_PASSWORD}' + '&phone={to}&json=1'),
            2: (f'{CALL_URL.SMSCENTRE_URL}?login={self.CALL_LOGIN}&psw={self.CALL_PASSWORD}' + '&phones={to}&mes=code&call=1&fmt=3'),
            3: (f'{CALL_URL.UCALLER_URL}?key={self.CALL_PASSWORD}&service_id={self.CALL_LOGIN}' + '&phone={to}'),
        }
        return url_dict_call.get(key)

    def __get_sms_url(self, key: int) -> Optional[str]:
        url_dict_sms = {
            0: (f'{SMS_URL.SMSRU_URL}?api_id={self.SMS_API_ID}' + '&to={to}&msg={text}&json=1'),
            1: (f'{SMS_URL.INFOSMS_URL}?login={self.SMS_LOGIN}&pwd={self.SMS_PASSWORD}&sender={self.SMS_FROM}' + '&phones={to}&message={text}'),
            2: (f'{SMS_URL.IQSMS_URL}?login={self.SMS_LOGIN}&password={self.SMS_PASSWORD}' + '&phone={to}&text={text}'),
            3: (f'{SMS_URL.SMSCENTRE_URL}?login={self.SMS_LOGIN}&psw={self.SMS_PASSWORD}' + '&phones={to}&mes={text}'),
            4: (f'{SMS_URL.SMS_SENDING_URL}?login={self.SMS_LOGIN}&password={self.SMS_PASSWORD}' + '&txt={text}&to={to}'),
            5: (f'{SMS_URL.SMS_PROSTO_URL}?login={self.SMS_LOGIN}&password={self.SMS_PASSWORD}&method=push_msg&format=json&sender_name={self.SMS_FROM}' + 'text={text}&phone={to}&key={api_id}'),
            6: (f'{SMS_URL.WEBSZK_URL}?user={self.SMS_LOGIN}&pwd={self.SMS_PASSWORD}&sadr={self.SMS_FROM}' + '&text={text}&dadr={to}'),
        }
        return url_dict_sms.get(key)

    @classmethod
    def call_url(cls, url_type: int) -> Optional[str]:
        return cls().__get_call_url(url_type)

    @classmethod
    def sms_url(cls, url_type: int) -> Optional[str]:
        return cls().__get_sms_url(url_type)



import pytest

from garpix_notify.models import SystemNotify
from .utils.common_class import CommonTestClass
from .utils.generate_data import (
    generate_users, generate_category, generate_system_notify_data, generate_system_template_data,
    generate_compiled_system
)


@pytest.mark.django_db
class TestSystemNotify(CommonTestClass):

    @pytest.fixture
    def setup(self):
        # Tests Events
        self.PASS_TEST_EVENT = 1
        self.PASS_TEST_EVENT_1 = 2

        # Users data
        self.data_user = generate_users(1)[0]
        self.user_list = self.create_system_user_list()
        self.user_1 = self.create_user(self.data_user)

        # Data for test user
        self.some_data_dict = generate_system_notify_data()
        self.data_category = generate_category()[0]

        self.data_template_system_1 = generate_system_template_data(self.some_data_dict, self.PASS_TEST_EVENT)[0]
        self.data_compiled_system_1 = generate_compiled_system(self.some_data_dict, self.PASS_TEST_EVENT, self.user_1)[0]

        self.data_template_email_2 = generate_system_template_data(self.some_data_dict, self.PASS_TEST_EVENT_1)[0]

        # Creating templates/category
        self.category_1 = self.create_category(self.data_category)
        self.template_1 = self.create_template(self.data_template_system_1, self.category_1)
        self.template_2 = self.create_template(self.data_template_email_2, self.category_1, self.user_list)

    def test_system_notify_positive(self, setup):
        assert self.category_1.title == self.data_category['title']
        assert self.category_1.template == self.data_category['template']
        assert self.template_1.title == self.data_template_system_1['title']
        assert self.template_1.subject, self.data_template_system_1['subject']
        assert self.template_1.text == self.data_template_system_1['text']
        assert self.template_1.html == self.data_template_system_1['html']
        assert self.template_1.type == self.data_template_system_1['type']
        assert self.template_1.event == self.data_template_system_1['event']

        # Create system notify
        SystemNotify.send(
            self.some_data_dict, user=self.user_1, event=self.PASS_TEST_EVENT
        )
        notify = SystemNotify.objects.filter(user=self.user_1, event=self.PASS_TEST_EVENT).first()

        assert notify.event == self.template_1.event
        assert notify.title == self.data_compiled_system_1['title']
        assert notify.data_json == self.data_compiled_system_1['data_json']
        assert notify.type == self.data_compiled_system_1['type']
        assert notify.event == self.data_compiled_system_1['event']

    def test_system_notify_user_list(self, setup):
        assert self.category_1.title == self.data_category['title']
        assert self.category_1.template == self.data_category['template']
        assert self.template_1.title == self.data_template_system_1['title']
        assert self.template_1.subject, self.data_template_system_1['subject']
        assert self.template_1.text == self.data_template_system_1['text']
        assert self.template_1.html == self.data_template_system_1['html']
        assert self.template_1.type == self.data_template_system_1['type']
        assert self.template_1.event == self.data_template_system_1['event']

        # Create system notify
        SystemNotify.send(self.some_data_dict, event=self.PASS_TEST_EVENT_1)
        notify_qs = SystemNotify.objects.filter(event=self.PASS_TEST_EVENT_1)

        for notify in notify_qs:
            self.data_compiled_system_2 = (
                generate_compiled_system(self.some_data_dict, self.PASS_TEST_EVENT_1, notify.user)[0]
            )

            assert notify.event == self.template_2.event
            assert notify.title == self.data_compiled_system_2['title']
            assert notify.data_json == self.data_compiled_system_2['data_json']
            assert notify.type == self.data_compiled_system_2['type']
            assert notify.event == self.data_compiled_system_2['event']



from django.conf import settings
from django.utils.timezone import now
from django.db import DatabaseError, ProgrammingError

from viberbot import Api, BotConfiguration
from viberbot.api.messages import TextMessage

from garpix_notify.models.config import NotifyConfig
from garpix_notify.models.choices import STATE
from garpix_notify.utils import ReceivingUsers


class ViberClient:

    def __init__(self, notify):
        self.notify = notify
        try:
            self.config = NotifyConfig.get_solo()
            self.IS_VIBER_ENABLED = self.config.is_viber_enabled
            self.VIBER_API_KEY = self.config.viber_api_key
            self.VIBER_BOT_NAME = self.config.viber_bot_name
        except (DatabaseError, ProgrammingError):
            self.IS_VIBER_ENABLED = getattr(settings, 'IS_VIBER_ENABLED', True)
            self.VIBER_API_KEY = getattr(settings, 'VIBER_API_KEY', '000000000:AAAAAAAAAA-AAAAAAAA-_AAAAAAAAAAAAAA')
            self.VIBER_BOT_NAME = getattr(settings, 'VIBER_BOT_NAME', 'MySuperBot')

    def __send_viber_client(self):
        if not self.IS_VIBER_ENABLED:
            self.notify.state = STATE.DISABLED
            self.notify.to_log('Not sent (sending is prohibited by settings)')
            return

        text = self.notify.text
        users_list = self.notify.users_list.all().order_by('-mail_to_all')

        viber = Api(BotConfiguration(
            name=self.VIBER_BOT_NAME,
            avatar='',
            auth_token=self.VIBER_API_KEY
        ))
        try:
            result = False
            if users_list.exists():
                participants: list = ReceivingUsers.run_receiving_users(users_list, 'viber_chat_id')
                if participants:
                    for participant in participants:
                        result = viber.send_messages(to=participant, messages=[TextMessage(text=text)])
                # если не добавлены, сообщение приходит пользователю(получателю) или тому, кто указан в коде
            else:
                result = viber.send_messages(to=self.notify.viber_chat_id, messages=[TextMessage(text=text)])
            if result:
                self.notify.state = STATE.DELIVERED
                self.notify.sent_at = now()
            else:
                self.notify.state = STATE.REJECTED
                self.notify.to_log('REJECTED WITH DATA, please test it.')
        except Exception as e:
            self.notify.state = STATE.REJECTED
            self.notify.to_log(str(e))

    @classmethod
    def send_viber(cls, notify):
        cls(notify).__send_viber_client()



import json
from django.http import HttpResponse
from django.views.decorators.csrf import csrf_exempt
from viberbot import Api, BotConfiguration
from viberbot.api.messages import TextMessage
from .models.config import NotifyConfig
from django.contrib.auth import get_user_model
from django.urls import reverse

User = get_user_model()


# # установка вебхука
def send_webhook(request):
    config = NotifyConfig.get_solo()
    viber = Api(BotConfiguration(
        name=config.viber_bot_name,
        avatar='',
        auth_token=config.viber_api_key
    ))
    current_host = request.META['HTTP_HOST']
    print(reverse("viber_check_webhook"), 'reverse(viber_check_webhook)')
    viber.set_webhook(url=f'https://{current_host}' + reverse('viber_check_webhook'),
                      webhook_events=['unsubscribed', 'conversation_started', 'message', 'seen', 'delivered'])


# проверка ивентов
@csrf_exempt
def viber_check_webhook(request):
    if request.method == "POST":
        response = json.loads(request.body.decode('utf-8'))  # получение колбэка
        if response['event'] == 'conversation_started':
            print("Приветствую пользователя")
            conversation(response)
        if response['event'] == 'subscribed':
            print("subscribed event")
            subscribed_event(response)
        if response['event'] == 'message':
            print("message event")
            add_viber_user(response)
        if response['event'] == 'webhook':
            print(response)
            print("Webhook успешно установлен")
            return HttpResponse(status=200)
        return HttpResponse(status=200)  # всегда возвращает 200


# отправка приветственного сообщения
def conversation(response):
    config = NotifyConfig.get_solo()
    viber = Api(BotConfiguration(
        name=config.viber_bot_name,
        avatar='',
        auth_token=config.viber_api_key
    ))
    id_user = response['user']['id']
    user_name = response['user']['name']
    print(id_user, ' id in conversation')
    if response['subscribed'] == 'false':
        print(response, 'response in conversation')
        viber.send_messages(to=id_user, messages=[
            TextMessage(text=f'Привет, {user_name}, {config.viber_welcome_text}')])


# доступ к получению сообщений от бота по viber_secret_key
def add_viber_user(response):
    config = NotifyConfig.get_solo()
    viber = Api(BotConfiguration(
        name=config.viber_bot_name,
        avatar='',
        auth_token=config.viber_api_key
    ))
    secret_key = response['message']['text']  # из ответа вайбера берет данные о юзере
    viber_user_id = response['sender']['id']
    viber_user_name = response['sender']['name']
    viber_user = User.objects.filter(viber_secret_key=secret_key).first()  # ищем юзера с верным сикрет кеем
    if viber_user is not None and viber_user.viber_secret_key == secret_key:
        viber_user.viber_chat_id = viber_user_id
        viber_user.save()
        viber.send_messages(to=viber_user_id, messages=[
            TextMessage(text=f'{viber_user_name}, {config.viber_success_added_text}')])
    else:
        viber.send_messages(to=viber_user_id,
                            messages=[TextMessage(text=f'{viber_user_name}, {config.viber_failed_added_text}')])


# ответ на подписку
def subscribed_event(response):
    config = NotifyConfig.get_solo()
    viber = Api(BotConfiguration(
        name=config.viber_bot_name,
        avatar='',
        auth_token=config.viber_api_key
    ))
    id_user = response['user']['id']
    user_name = response['user']['name']
    viber.send_messages(to=id_user, messages=[
        TextMessage(text=f'{user_name},  {config.viber_text_for_new_sub}')])



import json
from copy import copy

from asgiref.sync import async_to_sync
from channels.layers import get_channel_layer

from django.conf import settings
from django.utils import timezone
from django.db.models import Manager
from django.db import models, transaction
from django.contrib.auth import get_user_model
from django.utils.html import format_html
from django.utils.module_loading import import_string

from .system_log import SystemNotifyErrorLog
from .choices import TYPE, STATE
from ..utils import ReceivingUsers
from .template import NotifyTemplate
from ..exceptions import IsInstanceException, DataTypeException, ArgumentsEmptyException, UsersListIsNone

SystemNotifyMixin = import_string(getattr(settings, 'GARPIX_SYSTEM_NOTIFY_MIXIN', 'garpix_notify.mixins.notify_mixin.NotifyMixin'))

User = get_user_model()


class SystemNotify(SystemNotifyMixin, models.Model):
    """
    Системное уведомление
    """
    title = models.CharField('Название', max_length=255, default='', blank=True)
    user: User = models.ForeignKey(settings.AUTH_USER_MODEL, blank=True, null=True, on_delete=models.SET_NULL,
                                   related_name='system_notifies', verbose_name='Пользователь (получатель)')

    state = models.IntegerField('Состояние', choices=STATE.CHOICES, default=STATE.WAIT)
    event = models.IntegerField('Событие', choices=settings.CHOICES_NOTIFY_EVENT, blank=True, null=True)
    room_name = models.CharField('Название комнаты', max_length=255, null=True, blank=True)

    type = models.IntegerField('Тип', default=TYPE.SYSTEM, choices=TYPE.CHOICES, blank=True)
    data_json = models.JSONField('Данные JSON', blank=True, null=True, default=dict)

    created_at = models.DateTimeField('Дата создания', auto_now_add=True)
    sent_at = models.DateTimeField('Дата отправки', blank=True, null=True)

    objects = Manager()

    def __str__(self):
        return self.title if self.title and self.title != '' else f'{self.user} - {self.room_name}'

    @staticmethod
    def send(data_json: dict, user: User = None, event: int = None, room_name: str = None,  # noqa: C901
             title: str = None, templates: list = None, notify_type: str = None, **kwargs) -> None:

        if user is None and event is None and templates is None:
            raise ArgumentsEmptyException

        if not isinstance(data_json, dict):
            raise DataTypeException(data_json, 'dict')

        if user and not isinstance(user, User):
            raise IsInstanceException

        if templates and not isinstance(templates, list):
            raise DataTypeException('templates', 'list')

        if notify_type is None:
            notify_type = getattr(settings, 'DEFAULT_SYSTEM_NOTIFY_TYPE', 'system')

        system_notify_user_list: list = []
        notify_json: dict = data_json

        if event and templates is None:
            templates = list(
                NotifyTemplate.objects
                .filter(event=event, is_active=True, type=TYPE.SYSTEM)
                .values_list('id', flat=True)
            )

        if templates:
            templates_instances = (
                NotifyTemplate.objects
                .select_related('user')
                .prefetch_related('user_lists')
                .filter(id__in=templates, is_active=True, type=TYPE.SYSTEM)
            )
            if templates_instances.exists():
                for template in templates_instances:

                    if template.user:
                        system_notify_user_list.append(template.user)

                    if template.subject:
                        title = template.subject

                    if template.user_lists.exists():
                        notify_users_list = template.user_lists.all()
                        notify_users_list: list = ReceivingUsers.run_receiving_users(notify_users_list, 'user')
                        system_notify_user_list.extend([
                            user for user in notify_users_list if user
                        ])

        notify_json['type'] = notify_type

        if event:
            notify_json['event_id'] = event

        if user:
            system_notify_user_list.append(user)

        if not system_notify_user_list:
            raise UsersListIsNone

        for notify_user in system_notify_user_list:

            user_pk: int = notify_user.pk
            notify_json['user'] = user_pk

            notify_room_name = f'room_{user_pk}' if not room_name else room_name
            notify_title = f'{notify_user} - {notify_room_name}' if not title else title

            instance = SystemNotify.objects.create(
                title=notify_title,
                user=notify_user,
                event=event,
                room_name=notify_room_name,
                data_json=notify_json,
                **kwargs
            )

            if instance:
                transaction.on_commit(lambda: instance.send_notification())

    def send_notification(self):
        try:
            notify_dict = copy(self.data_json)
            if isinstance(notify_dict, str):
                notify_dict = json.loads(notify_dict)

            notify_dict['id'] = self.id

            if self.room_name:
                group_name = self.room_name
            else:
                group_name = f'room_{self.user.id}'

            async_to_sync(get_channel_layer().group_send)(
                group_name,
                notify_dict
            )
            self.state = STATE.DELIVERED
            self.sent_at = timezone.now()
        except Exception as e:
            self.state = STATE.REJECTED
            self.to_log(str(e))
        self.save(update_fields=['state', 'sent_at'])

    def to_log(self, error_text) -> None:
        log: SystemNotifyErrorLog = SystemNotifyErrorLog(notify=self, error=error_text)
        log.save()

    def get_format_state(self):
        if self.state == STATE.WAIT:
            return format_html('<span style="color:orange;">В ожидании</span>')
        elif self.state == STATE.DELIVERED:
            return format_html('<span style="color:green;">Отправлено</span>')
        elif self.state == STATE.REJECTED:
            return format_html('<span style="color:red;">Отклонено</span>')
        elif self.state == STATE.DISABLED:
            return format_html('<span style="color:red;">Отправка запрещена</span>')
        return format_html('<span style="color:black;">Неизвестный статус</span>')

    get_format_state.short_description = 'Статус'

    class Meta:
        verbose_name = 'Ситемное уведомление'
        verbose_name_plural = 'Системные уведомления'



from django.db import models
from django.db.models import Manager


class NotifyCategory(models.Model):
    title = models.CharField(max_length=255, verbose_name='Заголовок')
    template = models.TextField(verbose_name='Шаблон', default='{{text}}')
    created_at = models.DateTimeField(auto_now_add=True, verbose_name='Дата создания')

    objects = Manager()

    def __str__(self):
        return self.title

    class Meta:
        verbose_name = 'Категория'
        verbose_name_plural = 'Категории'



from django.urls import path
from .views import send_webhook, viber_check_webhook

urlpatterns = [
    path('send_webhook', send_webhook, name='send_webhook'),
    path('viber_check_webhook', viber_check_webhook, name='viber_check_webhook'),
]



import requests
from django.conf import settings
from django.db import DatabaseError, ProgrammingError
from django.utils.timezone import now

from garpix_notify.models.config import NotifyConfig
from garpix_notify.models.choices import STATE, SMS_URL
from garpix_notify.utils.receiving import ReceivingUsers
from garpix_notify.utils.send_data import SendData


class SMSClient:

    def __init__(self, notify):
        self.notify = notify
        try:
            self.config = NotifyConfig.get_solo()
            self.IS_SMS_ENABLED = self.config.is_sms_enabled
            self.SMS_URL_TYPE = self.config.sms_url_type
        except (DatabaseError, ProgrammingError):
            self.IS_SMS_ENABLED = getattr(settings, 'IS_SMS_ENABLED', True)
            self.SMS_URL_TYPE = getattr(settings, 'SMS_URL_TYPE', 0)

    def __sms_ru_client(self, response: dict) -> None:
        if response['status'] == 'OK':
            self.notify.to_log(
                f"Статус основного запроса: {response['status']}, Код статуса: \
                {response['status_code']}, Баланс: {response['balance']}")
            for key in response['sms']:
                if response['sms'][key]['status'] == 'ERROR':
                    self.notify.to_log(
                        f"Ошибка у абонента: Номер: {key}, Статус: {response['sms'][key]['status']}, \
                        Код статуса: {response['sms'][key]['status_code']}, Описание: \
                        {response['sms'][key]['status_text']}")
            self.notify.state = STATE.DELIVERED
            self.notify.sent_at = now()
        else:
            self.notify.to_log(
                f"Статус: {response['status']}, Код статуса: {response['status_code']}, "
                f"Описание ошибки: {response['status_text']}")
            self.notify.state = STATE.REJECTED

    def __web_szk_client(self, response) -> None:
        try:
            int(response.text)
            self.notify.state = STATE.DELIVERED
            self.notify.sent_at = now()
        except Exception as e:
            self.notify.to_log(str(e))
            self.notify.state = STATE.REJECTED

    def __iq_sms_client(self, response: dict) -> None:
        if response['status'] == 'ok':
            self.notify.to_log(
                f"Статус: {response['status']}, Код статуса: {response['code']}, Описание: \
                                   {response['description']}")
            self.notify.state = STATE.DELIVERED
            self.notify.sent_at = now()
        else:
            self.notify.to_log(
                f"Статус: {response['status']}, Код статуса: {response['code']}, "
                f"Описание ошибки: {response['description']}")
            self.notify.state = STATE.REJECTED

    def __sms_sending_client(self, response: dict) -> None:
        if response['code'] == 1:
            self.notify.to_log(
                f"Статус: {response['code']}, Описание: {response['descr']}")
            self.notify.state = STATE.DELIVERED
            self.notify.sent_at = now()
        else:
            self.notify.to_log(
                f"Статус: {response['code']}, Описание ошибки: {response['descr']}")
            self.notify.state = STATE.REJECTED

    def __sms_prosto_client(self, response: dict) -> None:
        if response['response']['msg']['err_code'] == 0:
            self.notify.to_log(
                f"Статус: {response['response']['msg']['err_code']}, Описание: \
                {response['response']['msg']['text']}")
            self.notify.state = STATE.DELIVERED
            self.notify.sent_at = now()
        else:
            self.notify.to_log(
                f"Статус: {response['response']['msg']['err_code']}, Описание ошибки: \
                {response['response']['msg']['text']}")
            self.notify.state = STATE.REJECTED

    def __send_sms(self):  # noqa

        if not self.IS_SMS_ENABLED:
            self.notify.state = STATE.DISABLED
            self.notify.to_log('Not sent (sending is prohibited by settings)')
            return

        try:
            users_list = self.notify.users_list.all().order_by('-mail_to_all')
            if not users_list.exists():
                phones: str = self.notify.phone
            else:
                phones_list: list = ReceivingUsers.run_receiving_users(users_list, 'phone')
                phones = ','.join(phones_list)
            msg = str(self.notify.text.replace(' ', '+'))

            url = SendData.sms_url(self.SMS_URL_TYPE).format(to=phones, text=msg)
            response = requests.get(url)
            response_dict = response.json()

            if self.SMS_URL_TYPE == SMS_URL.SMSRU_ID:
                self.__sms_ru_client(response_dict)

            elif self.SMS_URL_TYPE == SMS_URL.WEBSZK_ID:
                self.__web_szk_client(response)

            elif self.SMS_URL_TYPE == SMS_URL.IQSMS_ID:
                self.__iq_sms_client(response_dict)

            elif self.SMS_URL_TYPE == SMS_URL.INFOSMS_ID:
                self.notify.state = STATE.DELIVERED
                self.notify.sent_at = now()

            elif self.SMS_URL_TYPE == SMS_URL.SMSCENTRE_ID:
                self.notify.state = STATE.DELIVERED
                self.notify.sent_at = now()

            elif self.SMS_URL_TYPE == SMS_URL.SMS_SENDING_ID:
                self.__sms_sending_client(response_dict)

            elif self.SMS_URL_TYPE == SMS_URL.SMS_PROSTO_ID:
                self.__sms_prosto_client(response_dict)

        except Exception as e:
            self.notify.state = STATE.REJECTED
            self.notify.to_log(str(e))

    @classmethod
    def send_sms(cls, notify):
        cls(notify).__send_sms()



from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group
from django.utils.crypto import get_random_string

from garpix_notify.models import NotifyCategory, NotifyTemplate, NotifyUserList, NotifyUserListParticipant


class CommonTestClass:

    def create_user(self, user_data: dict):
        user = get_user_model().objects.create_user(**user_data)
        return user

    def create_category(self, category_data: dict):
        category = NotifyCategory.objects.create(**category_data)
        return category

    def create_template(self, template_data: dict, category: NotifyCategory, user_list: NotifyUserList = None):
        template = NotifyTemplate.objects.create(category=category, **template_data)

        if user_list:
            template.user_lists.add(user_list)
            template.save()

        return template

    def create_user_list(self):
        user_list = NotifyUserList.objects.create(title='userlist_' + get_random_string(length=4))
        group = Group.objects.create(name='group_' + get_random_string(length=4))
        user_list.user_groups.add(group)

        user_list_participant_data: list = [{
            'user_list': user_list,
            'email': f'{get_random_string(length=5)}@garpix.com'
        } for _ in range(5)]

        for participant in user_list_participant_data:
            NotifyUserListParticipant.objects.create(**participant)

        return user_list

    def create_system_user_list(self):
        user_list = NotifyUserList.objects.create(title='userlist_' + get_random_string(length=4))
        group = Group.objects.create(name='group_' + get_random_string(length=4))
        user_list.user_groups.add(group)

        # Participant data
        user_list_participant_data = [
            {'user_list': user_list, 'email': 'test2@garpix.com'},
            {'user_list': user_list}
        ]
        for user_list_data in user_list_participant_data:
            NotifyUserListParticipant.objects.create(**user_list_data)

        # Create users
        user_model = get_user_model()
        user_data_list = [
            {
                'username': f'user_{user_count}_' + get_random_string(length=3),
                'email': f'{get_random_string(length=5).capitalize()}@garpix.com',
                'password': f'1{get_random_string(length=9)}1',
                'first_name': get_random_string(length=5),
                'last_name': get_random_string(length=5),
            } for user_count in range(10)
        ]
        for user_data in user_data_list:
            user = user_model.objects.create_user(**user_data)
            user.groups.add(group)
            user.save()

        return user_list



from django.contrib import admin
from solo.admin import SingletonModelAdmin
from ..models.config import NotifyConfig


@admin.register(NotifyConfig)
class NotifyConfigAdmin(SingletonModelAdmin):
    fieldsets = (
        (None, {
            'fields': ('periodic', 'is_email_enabled', 'is_sms_enabled', 'is_call_enabled', 'is_push_enabled',
                       'is_telegram_enabled', 'is_viber_enabled', 'is_whatsapp_enabled')
        }),
        ('Почта', {
            'fields': ('email_max_day_limit', 'email_max_hour_limit', 'email_malling')
        }),
        ('СМС', {
            'fields': ('sms_url_type', 'sms_api_id', 'sms_login', 'sms_password', 'sms_from')
        }),
        ('Звонки', {
            'fields': ('call_url_type', 'call_api_id', 'call_login', 'call_password')
        }),
        ('Telegram', {
            'fields': ('telegram_api_key', 'telegram_bot_name', 'telegram_welcome_text', 'telegram_help_text',
                       'telegram_bad_command_text', 'telegram_success_added_text', 'telegram_failed_added_text',
                       'telegram_parse_mode', 'telegram_disable_notification', 'telegram_disable_web_page_preview',
                       'telegram_allow_sending_without_reply', 'telegram_timeout')
        }),
        ('Viber', {
            'fields': ('viber_api_key', 'viber_bot_name', 'viber_success_added_text',
                       'viber_failed_added_text', 'viber_text_for_new_sub', 'viber_welcome_text')
        }),
        ('WhatsApp (Twilio)', {
            'fields': ('whatsapp_sender', 'whatsapp_auth_token', 'whatsapp_account_sid')
        }),
    )

    class Media:
        css = {
            'all': ('css/admin/garpix_notify.css', )
        }
        js = ('js/admin/garpix_notify.js',)



from django.conf import settings
from django.utils.timezone import now
from django.db import DatabaseError, ProgrammingError

from garpix_notify.models.config import NotifyConfig
from garpix_notify.models.choices import STATE


class TelegramClient:

    def __init__(self, notify):
        self.notify = notify
        try:
            self.config = NotifyConfig.get_solo()
            self.IS_TELEGRAM_ENABLED = self.config.is_telegram_enabled
            self.TELEGRAM_API_KEY = self.config.telegram_api_key
            self.TELEGRAM_PARSE_MODE = self.config.telegram_parse_mode
            self.TELEGRAM_DISABLE_NOTIFICATION = self.config.telegram_disable_notification
            self.TELEGRAM_DISABLE_PAGE_PREVIEW = self.config.telegram_disable_web_page_preview
            self.TELEGRAM_SENDING_WITHOUT_REPLY = self.config.telegram_allow_sending_without_reply
            self.TELEGRAM_TIMEOUT = self.config.telegram_timeout
        except (DatabaseError, ProgrammingError):
            self.IS_TELEGRAM_ENABLED = True
            self.TELEGRAM_PARSE_MODE = getattr(settings, 'TELEGRAM_PARSE_MODE', None)
            self.TELEGRAM_DISABLE_NOTIFICATION = getattr(settings, 'TELEGRAM_DISABLE_NOTIFICATION', False)
            self.TELEGRAM_DISABLE_PAGE_PREVIEW = getattr(settings, 'TELEGRAM_DISABLE_PAGE_PREVIEW', False)
            self.TELEGRAM_SENDING_WITHOUT_REPLY = getattr(settings, 'TELEGRAM_SENDING_WITHOUT_REPLY', False)
            self.TELEGRAM_TIMEOUT = getattr(settings, 'TELEGRAM_TIMEOUT', None)
            self.TELEGRAM_API_KEY = getattr(settings, 'TELEGRAM_API_KEY', '000000000:AAAAAAAAAA-AAAAAAAA-_AAAAAAAAAAAAAA')

    def __chunks(self, s, n):
        """Produce `n`-character chunks from `s`."""
        for start in range(0, len(s), n):
            yield s[start:start + n]

    def __send_telegram_client(self):
        import telegram

        if not self.IS_TELEGRAM_ENABLED:
            self.notify.state = STATE.DISABLED
            self.notify.to_log('Not sent (sending is prohibited by settings)')
            return

        bot = telegram.Bot(token=self.TELEGRAM_API_KEY)
        parse_mode = self.TELEGRAM_PARSE_MODE if self.TELEGRAM_PARSE_MODE != '' else None

        try:
            result = False
            for chunk in self.__chunks(str(self.notify.text), 4096):
                result = bot.sendMessage(chat_id=self.notify.telegram_chat_id,
                                         text=chunk,
                                         parse_mode=parse_mode,
                                         disable_web_page_preview=self.TELEGRAM_DISABLE_PAGE_PREVIEW,
                                         disable_notification=self.TELEGRAM_DISABLE_NOTIFICATION,
                                         timeout=self.TELEGRAM_TIMEOUT,
                                         allow_sending_without_reply=self.TELEGRAM_SENDING_WITHOUT_REPLY)
            if result:
                self.notify.state = STATE.DELIVERED
                self.notify.sent_at = now()
            else:
                self.notify.state = STATE.REJECTED
                self.notify.to_log('REJECTED WITH DATA, please test it.')
        except Exception as e:  # noqa
            self.notify.state = STATE.REJECTED
            self.notify.to_log(str(e))

    @classmethod
    def send_telegram(cls, notify):
        cls(notify).__send_telegram_client()



from typing import Optional

from django.contrib.auth import get_user_model


class ReceivingUsers:

    def __init__(self, users_list: list, value: Optional[str] = None):
        self.receivers = []
        self.users_list = users_list
        self.value = value

    def __returning_specific_list(self, receivers: list) -> list:
        receivers_list = list(set(map(lambda x: x.get(self.value), receivers)))
        return receivers_list

    def __forming_data_list(self, queryset) -> list:
        data_list = [{'user': user,
                      'email': user.email,
                      'phone': user.phone,
                      'viber_chat_id': user.viber_chat_id,
                      'telegram_chat_id': user.telegram_chat_id} for user in queryset]
        return data_list

    def __receiving_users(self) -> list:  # noqa: C901
        user_model = get_user_model()

        for user_list in self.users_list:
            # Массовая рассылка по всем пользователям сайта
            if user_list.mail_to_all:
                users = user_model.objects.all()
                data_list: list = self.__forming_data_list(users)
                self.receivers.extend(data_list)

                if self.value:
                    return self.__returning_specific_list(self.receivers)
                return self.receivers

            # Выполняем запросы
            users_participants = user_list.participants.all()
            user_groups_qs = user_list.user_groups.all()
            users_qs = user_list.users.all()

            # Собираем данные из дополнительного списка получателей, если он есть.
            if users_participants.exists():
                self.receivers.extend([{
                    'user': participant.user,
                    'email': participant.user.email if participant.user else participant.email,
                    'phone': participant.user.phone if participant.user else participant.phone,
                    'viber_chat_id': (
                        participant.user.viber_chat_id if participant.user else participant.viber_chat_id
                    ),
                    'telegram_chat_id': (
                        participant.user.telegram_chat_id if participant.user else participant.telegram_chat_id
                    ),
                } for participant in users_participants])

            # Собираем данные из групп, которые входят в список рассылки
            if user_groups_qs.exists():
                group_users = user_model.objects.prefetch_related('groups').filter(groups__in=user_groups_qs)
                if group_users.exists():
                    data_list: list = self.__forming_data_list(group_users)
                    self.receivers.extend(data_list)

            # Собираем данные по пользователям, если они были переданы
            if users_qs.exists():
                data_list: list = self.__forming_data_list(users_qs)
                self.receivers.extend(data_list)

        if self.value:
            return self.__returning_specific_list(self.receivers)

        return self.receivers

    @classmethod
    def run_receiving_users(cls, users_list: list, value: str = None) -> list:
        return cls(users_list, value).__receiving_users()



from django.conf import settings
from django.utils.timezone import now
from django.db import DatabaseError, ProgrammingError

from twilio.rest import Client

from garpix_notify.models.config import NotifyConfig
from garpix_notify.models.choices import STATE
from garpix_notify.utils import ReceivingUsers


class WhatsAppClient:

    def __init__(self, notify):
        self.notify = notify
        try:
            self.config = NotifyConfig.get_solo()
            self.IS_WHATS_APP_ENABLED = self.config.is_whatsapp_enabled
            self.WHATS_APP_AUTH_TOKEN = self.config.whatsapp_auth_token
            self.WHATS_APP_ACCOUNT_SID = self.config.whatsapp_account_sid
            self.WHATS_APP_NUMBER_SENDER = self.config.whatsapp_sender
        except (DatabaseError, ProgrammingError):
            self.IS_WHATS_APP_ENABLED = getattr(settings, 'IS_WHATS_APP_ENABLED', True)
            self.WHATS_APP_AUTH_TOKEN = getattr(settings, 'WHATS_APP_AUTH_TOKEN', None)
            self.WHATS_APP_ACCOUNT_SID = getattr(settings, 'WHATS_APP_ACCOUNT_SID', None)
            self.WHATS_APP_NUMBER_SENDER = getattr(settings, 'WHATS_APP_NUMBER_SENDER', '')

    def __send_message(self):
        if not self.IS_WHATS_APP_ENABLED:
            self.notify.state = STATE.DISABLED
            self.notify.to_log('Not sent (sending is prohibited by settings)')
            return

        client = Client(self.WHATS_APP_ACCOUNT_SID, self.WHATS_APP_AUTH_TOKEN)
        text_massage = self.notify.text
        users_list = self.notify.users_list.all().order_by('-mail_to_all')

        result = False

        try:
            if users_list.exists():
                participants: list = ReceivingUsers.run_receiving_users(users_list, 'phone')
                if participants:
                    for participant in participants:
                        result = client.messages.create(body=text_massage,
                                                        from_=f'whatsapp:{self.WHATS_APP_NUMBER_SENDER}',
                                                        to=f'whatsapp:{participant}')
            else:
                result = client.messages.create(body=text_massage,
                                                from_=f'whatsapp:{self.WHATS_APP_NUMBER_SENDER}',
                                                to=f'whatsapp:{self.notify.phone}')
            if result.sid:
                self.notify.state = STATE.DELIVERED
                self.notify.sent_at = now()
            else:
                self.notify.state = STATE.REJECTED
                self.notify.to_log('REJECTED WITH DATA, please test it.')
        except Exception as e:
            self.notify.state = STATE.REJECTED
            self.notify.to_log(str(e))

    @classmethod
    def send_whatsapp(cls, notify):
        cls(notify).__send_message()



from django.conf import settings
from django.utils import timezone
from django.utils.module_loading import import_string
from asgiref.sync import async_to_sync
from channels.layers import get_channel_layer

from garpix_notify.models.choices import STATE, TYPE
from garpix_notify.models.config import NotifyConfig
from garpix_notify.models.notify import Notify


celery_app = import_string(settings.GARPIX_NOTIFY_CELERY_SETTINGS)

try:
    config = NotifyConfig.get_solo()
    PERIODIC_SENDING = config.periodic
except Exception:
    PERIODIC_SENDING = getattr(settings, 'PERIODIC_SENDING', 60)


@celery_app.task
def send_notifications():
    notifies = Notify.objects.filter(state__in=[STATE.WAIT]).exclude(type=TYPE.SYSTEM)

    for notify in notifies.iterator():
        if notify.state == STATE.WAIT:
            if notify.send_at is None:
                notify.start_send()
            else:
                if timezone.now() > notify.send_at:
                    notify.start_send()
    return


@celery_app.task
def send_system_notifications(notify_pk):
    instance = Notify.objects.get(pk=notify_pk)
    try:
        if instance.room_name:
            group_name = instance.room_name
        else:
            group_name = f'room_{instance.user.id}'
        async_to_sync(get_channel_layer().group_send)(
            group_name,
            {
                'id': notify_pk,
                'type': 'send_notify',
                'event': instance.event,
                'message': instance.html,
                'json_data': instance.data_json,
            }
        )
        instance.state = STATE.DELIVERED
        instance.sent_at = timezone.now()
    except Exception as e:  # noqa
        instance.state = STATE.REJECTED
        instance.to_log(str(e))
    instance.save()


celery_app.conf.beat_schedule.update({
    'periodic_task': {
        'task': 'garpix_notify.tasks.tasks.send_notifications',
        'schedule': PERIODIC_SENDING,
    }
})
celery_app.conf.timezone = 'UTC'



from django.contrib import admin
from ..models.file import NotifyFile


@admin.register(NotifyFile)
class NotifyFileAdmin(admin.ModelAdmin):
    list_display = ('file', 'created_at')
    readonly_fields = ('created_at',)



from setuptools import setup, find_packages
from os import path

here = path.join(path.abspath(path.dirname(__file__)), 'garpix_notify')

with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='garpix_notify',
    version='5.12.5',
    description='',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/garpixcms/garpix_notify',
    author='Garpix LTD',
    author_email='info@garpix.com',
    license='MIT',
    packages=find_packages(exclude=['testproject', 'testproject.*']),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'Django >= 1.11',
        'django-solo >= 1.1.3',
        'fcm-django == 0.3.11',
        'requests >= 2.18.4',
        'django-ckeditor >= 5.6.1',
        'python-telegram-bot >= 12.6.1',
        'viberbot >= 1.0.11',
        'django-uuslug >= 1.2.0',
        'Pillow >= 8.2.0',
        'celery>=4.4.2',
        'redis >= 3.5.3',
        'channels >= 3.0.3, <= 3.0.4',
        'channels-redis == 3.4.0',
        'asgiref >= 3.2.10, <= 3.3.4',
        'twilio == 7.10.0',
        'typing-extensions >= 4.3.0'
    ],
)



from django.db import models
import uuid


def generate_uuid():
    return uuid.uuid4().hex


TELEGRAM_CONNECT_USER_HELP = """Для получения уведомлений в Telegram вам необходимо:
1. Отправить боту https://t.me/{telegram_bot_name} сообщение /start:
2. Отправить ему сообщение /set {telegram_secret}
3. Все, вы будете получать уведомления.
"""


class UserNotifyMixin(models.Model):
    phone = models.CharField(max_length=30, blank=True, default='', verbose_name='Телефон')
    telegram_chat_id = models.CharField(max_length=200, blank=True, default='',
                                        verbose_name='Telegram ID пользователя/чата')
    telegram_secret = models.CharField(max_length=150, unique=True, default=generate_uuid,
                                       verbose_name='Ключ подключения Telegram')
    viber_chat_id = models.CharField(max_length=200, blank=True, default='',
                                     verbose_name='Viber ID пользователя/чата')
    viber_secret_key = models.CharField(max_length=200, blank=True, default='',
                                        verbose_name='Ключ подключения Viber')

    def get_telegram_connect_user_help(self):
        from ..models.config import NotifyConfig
        config = NotifyConfig.get_solo()
        return TELEGRAM_CONNECT_USER_HELP.format(
            telegram_secret=self.telegram_secret,
            telegram_bot_name=config.telegram_bot_name,
        )
    get_telegram_connect_user_help.short_description = 'Как подключить Telegram-уведомления'

    class Meta:
        abstract = True



import json
from asgiref.sync import async_to_sync
from channels.generic.websocket import WebsocketConsumer


class NotifyConsumer(WebsocketConsumer):
    def connect(self):
        self.user_id = self.scope['url_route']['kwargs']['user_id']
        self.room_group_name = f'room_{self.user_id}'
        async_to_sync(self.channel_layer.group_add)(
            self.room_group_name,
            self.channel_name
        )
        self.accept()

    def disconnect(self, close_code):
        async_to_sync(self.channel_layer.group_discard)(
            self.room_group_name,
            self.channel_name
        )

    def send_notify(self, event):
        message = event['message']
        self.send(text_data=json.dumps({
            'message': message
        }))



from typing import Optional, List

from django.db import models
from django.conf import settings
from django.db.models import Manager
from django.template import Template, Context

from .category import NotifyCategory
from .choices import TYPE
from .user_list import NotifyUserList

from ckeditor_uploader.fields import RichTextUploadingField
from ..mixins import UserNotifyMixin


class NotifyTemplate(UserNotifyMixin):
    title = models.CharField(max_length=255, verbose_name='Название для админа')
    subject = models.CharField(max_length=255, default='', blank=True, verbose_name='Заголовок')
    text = models.TextField(verbose_name='Текст')
    html = RichTextUploadingField(verbose_name='HTML')

    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.SET_NULL, blank=True, null=True,
                             verbose_name='Пользователь (получатель)')
    email = models.EmailField(max_length=255, blank=True, null=True, verbose_name='Email получатель',
                              help_text='Используется только в случае отсутствия указанного пользователя')

    type = models.IntegerField(choices=TYPE.CHOICES, verbose_name='Тип')
    category = models.ForeignKey(NotifyCategory, on_delete=models.CASCADE, related_name='templates', verbose_name='Категория')
    event = models.IntegerField(choices=settings.CHOICES_NOTIFY_EVENT, blank=True, null=True, verbose_name='Событие')

    created_at = models.DateTimeField(auto_now_add=True, verbose_name='Дата создания')
    is_active = models.BooleanField(default=True, verbose_name='Активный')
    send_at = models.DateTimeField(blank=True, null=True, verbose_name='Время начала отправки')

    user_lists = models.ManyToManyField(NotifyUserList, blank=True, verbose_name='Списки пользователей для рассылки')

    objects = Manager()

    def __str__(self):
        return self.title

    def render_subject(self, ct):
        template = Template(self.subject)
        context = Context(ct)
        return template.render(context)

    def render_text(self, ct):
        template = Template(self.text)
        context = Context(ct)
        return template.render(context)

    def render_html(self, ct):
        template = Template(self.html)
        context = Context(ct)
        return template.render(context)

    def get_event_data(self):
        return settings.NOTIFY_EVENTS[self.event]

    def get_context_description(self):
        try:
            text = self.get_event_data()['context_description'] if self.event else '--'
        except:  # noqa
            text = '--'
        return text
    get_context_description.short_description = 'Описание контекста шаблона'

    def get_event_description(self):
        try:
            text = self.get_event_data()['event_description'] if self.event else '--'
        except:  # noqa
            text = '--'
        return text
    get_event_description.short_description = 'Описание события'

    def get_test_data(self):
        try:
            data = self.get_event_data()['test_data'] if self.event else {}
        except:  # noqa
            data = {}
        return data

    @staticmethod
    def get_blank_events_message() -> Optional[str]:
        """ Метод возвращает сообщение о том, что есть Ивенты без шаблонов. """
        message: Optional[str] = None
        events = settings.NOTIFY_EVENTS.keys()
        notify_templates_events = NotifyTemplate.objects.filter(is_active=True).values_list('event', flat=True)
        events_without_templates: List[Optional[int]] = [
            settings.NOTIFY_EVENTS.get(event_id) for event_id in events if event_id not in notify_templates_events
        ]
        if events_without_templates:
            names_events_without_templates: str = ', '.join(
                map(lambda event: event.get('title'), events_without_templates)
            )
            message = 'Не найдено активных шаблонов для ивентов: ' + names_events_without_templates
        return message

    class Meta:
        verbose_name = 'Шаблон'
        verbose_name_plural = 'Шаблоны'
